PARSER_KEY = "parser"
LOADER_KEY = "loader"
SOURCE_KEY = "source"
SOURCE_ID_KEY = "id"
MAPPER_KEY = "mapper"
ENTRYPOINT_KEY = "entrypoint"
ARGS_KEY = "args"
STREAM_ID_KEY = "id"

PARSERS_GROUP = "parsers"
LOADERS_GROUP = "loaders"
MAPPERS_GROUP = "mappers"
FILTERS_GROUP = "filters"
DEFAULT_IO_LOADER_EP = "core.io"

# POSTPROCESS_GLOBAL_KEY = "__global__"
POSTPROCESS_TRANSFORMS = "transforms"
SCALER_STATISTICS = "scaler_statistics"
VECTOR_SCHEMA = "vector_schema"
VECTOR_SCHEMA_METADATA = "vector_schema_metadata"
