# Required identifier for this raw source. Contracts reference it under `source:`.
id: "{{ id }}"  # format: provider.dataset

# parser.entrypoint: registered parser name (not a file path)
parser:
  entrypoint: "{{ parser_ep }}"
  args:
{{ parser_args | to_yaml(4) }}

# loader.entrypoint: registered loader name (not a file path)
loader:
  entrypoint: "{{ loader_ep }}"
  args:
{{ loader_args | to_yaml(4) }}
